local game = {firstStart = true}

function game:load()
	if self.firstStart then
		self.mapX = 0
		self.mapY = 0
		mapscreenWidth = 1
		mapscreenHeight = 1
		self.mapBoxes = 1
		self.tileSize = 16

		--Win screen
		self.wonScreen = {
			color = color[1],
			alpha = 1
		}

		self.t = 0
		self.val = 0
		self.currentMap = 1
		self.firstStart = false
		self.totalMoves = 0
		self:reset()
	else
		love.resize(screen.width, screen.height)
	end
end

function game:reset(nextMap)
	nextMap = nextMap or false
	if self.won then nextMap = true end
	if nextMap then
		sound:play("swish", soundVolume, false, 0.8)
		self.currentMap = self.currentMap + 1
		if self.currentMap > #maps then
			self.currentMap = 1
		end
	else
		sound:play("tick", soundVolume)
	end
	self.map, self.mapWidth, self.mapHeight, self.mapBoxes = map:loadMap(self.currentMap)
	--Finding player start position
	local sx, sy = 1, 1
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local t = self.map[2][y][x]
			if t then
				if t.tile == 5 then
					sx = x
					sy = y
					break
				end
			end
		end
	end
	player:spawn(sx, sy)
	self.moves = 0
	self.won = false
	self.winSoundPlayed = false
	player.currentQuad = 3
	love.resize(screen.width, screen.height)
end

function game:formatMap()
	local boxes = 0
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local tile1 = self.map[1][y][x]
			local tile2 = self.map[2][y][x]
			if tile1 then
				if tile1 + 1 > 0 then
					self.map[1][y][x] = {
						tile = tile1 + 1,
						gridX = x - 1,
						gridY = y - 1,
						x = (x - 1) * self.tileSize,
						y = (y - 1) * self.tileSize
					}
				else 
					self.map[1][y][x] = false
				end
			end
			if tile2 then
				if tile2 + 1 > 0 then
					if tile2 + 1 == 4 then
						boxes = boxes + 1
					end
					self.map[2][y][x] = {
						tile = tile2 + 1,
						gridX = x - 1,
						gridY = y - 1,
						x = (x - 1) * self.tileSize,
						y = (y - 1) * self.tileSize
					}
				else
					self.map[2][y][x] = false
				end
			end
		end

	end
	self.mapBoxes = boxes
	game:scaleMap()
end

function game:scaleMap()
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local tile1 = self.map[1][y][x]
			local tile2 = self.map[2][y][x]
			if tile1 then
				tile1.x = (x - 1) * self.tileSize
				tile1.y = (y - 1) * self.tileSize
			end
			if tile2 then
				tile2.x = (x - 1) * self.tileSize
				tile2.y = (y - 1) * self.tileSize
			end
		end
	end
end

function game:update(dt)
	self.t = self.t + dt
	if self.t > math.pi * 2 then t = 0 end
	self.val = math.sin(self.t)
	player:update(dt)

	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local box = self.map[2][y][x]
			if box then
				box.x = box.x + ((box.gridX * self.tileSize) - box.x) * (smooth_factor * dt)
				box.y = box.y + ((box.gridY * self.tileSize) - box.y) * (smooth_factor * dt)
			end
		end
	end

	if self.won then
		self.wonScreen.alpha = self.wonScreen.alpha + (1 - self.wonScreen.alpha) * (smooth_factor * dt)
		if not self.winSoundPlayed then
			sound:play("swish", soundVolume, false, 0.6)
			sound:play("win", soundVolume)
			self.winSoundPlayed = true
		end
	else
		self.wonScreen.alpha = self.wonScreen.alpha + (0 - self.wonScreen.alpha) * (smooth_factor * dt)
	end
end

function game:draw()
	self.gameCanvas:clear()
	love.graphics.setCanvas(self.gameCanvas)
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local t = self.map[1][y][x]
			if t then
				love.graphics.setColor(255, 255, 255, 255)
				tile:draw(t.tile, self.mapX + t.x, self.mapY + t.y)
			end
		end
	end

	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local t = self.map[2][y][x]

			if t then
				love.graphics.setColor(255, 255, 255, 255)
				tile:draw(t.tile, self.mapX + t.x, self.mapY + t.y)
				if t.onTarget then
					love.graphics.setBlendMode("additive")
					love.graphics.setColor(255, 255, 255, 126)
					tile:draw(t.tile, self.mapX + t.x, self.mapY + t.y)
					love.graphics.setBlendMode("alpha")
				end
			end
		end
	end

	player:draw()

	if not self.won then
		love.graphics.setColor(color[4])
		love.graphics.setFont(font.small)
		love.graphics.printf("Moves: "..self.moves, 0, self.mapY - font.small:getHeight(), screen.width, "center")
		love.graphics.printf("Level: "..self.currentMap.."/"..#maps, 0, self.mapY + mapscreenHeight + 8, screen.width, "center")
	end

	self.textCanvas:clear()
	love.graphics.setCanvas(self.textCanvas)
	if self.wonScreen.alpha > 0 then

		love.graphics.setColor(color[4])
		setAlpha(self.wonScreen.alpha)
		love.graphics.setFont(font.medium)
		love.graphics.printf("You beat level "..self.currentMap.."!", 0, math.floor(screen.height * 0.15), screen.width, "center")
		love.graphics.setFont(font.medium)
		love.graphics.printf("Moves: "..self.moves, 0, math.floor(screen.height * 0.3), screen.width, "center")
		love.graphics.setFont(font.small)
		love.graphics.printf("Press 'r' to go to the next level", 0, math.floor(screen.height * 0.9), screen.width, "center")
	end
	love.graphics.setCanvas()

	--GAME
	love.graphics.setColor(255, 255, 255, 255)
	setAlpha(1 - self.wonScreen.alpha)
	love.graphics.draw(self.gameCanvas, (screen.width / 2), screen.height / 2, 0, 1 - self.wonScreen.alpha, 1 - self.wonScreen.alpha, screen.width / 2, screen.height / 2)

	--TEXT
	love.graphics.setColor(self.wonScreen.color)
	setAlpha(self.wonScreen.alpha)
	--love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

	love.graphics.setColor(255, 255, 255, 255 * self.wonScreen.alpha)
	love.graphics.draw(self.textCanvas, (screen.width / 2), screen.height / 2, 0, 2 - self.wonScreen.alpha, 2 - self.wonScreen.alpha, screen.width / 2, screen.height / 2)
end

function game:keypressed(key)
	if not console:getVisible() then
		if not self.won then
			if key == "w" or key == "up" then
				player:move("up")
			elseif key == "s" or key == "down" then
				player:move("down")
			elseif key == "d"  or key == "right" then
				player:move("right")
			elseif key == "a" or key == "left" then
				player:move("left")
			end
		end

		if key == "r" then
			self:reset()
		end
	end
end

function game:checkIfWon()
	local onTarget = 0
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local box = self.map[2][y][x]
			if box then
				if box.onTarget then
					onTarget = onTarget + 1
				end
			end
		end
	end
	if onTarget == self.mapBoxes then
		self.won = true
	end
end

function game:resize(w, h)
	mapscreenWidth = self.mapWidth * self.tileSize
	mapscreenHeight = self.mapHeight * self.tileSize
	self.mapX = math.floor((screen.width / 2) - (mapscreenWidth / 2))
	self.mapY = math.floor((screen.height / 2) - (mapscreenHeight / 2))
	local msaa = love.graphics.getSystemLimit("canvasfsaa")
	console:print(msaa)
	self.gameCanvas = love.graphics.newCanvas(screen.width, screen.height, "normal", msaa)
	self.textCanvas = love.graphics.newCanvas(screen.width, screen.height, "normal", msaa)
	player:spawn()
	game:scaleMap()
end

return game