local gameState = {states = {}, selectedState = false}

function gameState:newState(state)
	self.states[state] = require(state)
end

function gameState:getState()
	return self.selectedState
end

function gameState:selectState(state)
	if self.states[state] then
		if self.selectedState then
			if state ~= self.selectedState then gameState:unload() end
		end
		self.selectedState = state
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].load) == "function" then
				self.states[self.selectedState]:load()
			end
		end
	else
		error("State '"..state.."' not found")
	end
end

function gameState:load()
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].load) == "function" then
				self.states[self.selectedState]:load()
			end
		end
	end
end

function gameState:unload()
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].unload) == "function" then
				self.states[self.selectedState]:unload()
			end
		end
	end
end

function gameState:update(dt)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].update) == "function" then
				self.states[self.selectedState]:update(dt)
			end
		end
	end
end

function gameState:draw()
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].draw) == "function" then
				self.states[self.selectedState]:draw()
			end
		end
	end
end

function gameState:keypressed(key)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].keypressed) == "function" then
				self.states[self.selectedState]:keypressed(key)
			end
		end
	end
end

function gameState:keyreleased(key)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].keyreleased) == "function" then
				self.states[self.selectedState]:keyreleased(key)
			end
		end
	end
end

function gameState:textinput(t)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].textinput) == "function" then
				self.states[self.selectedState]:textinput(t)
			end
		end
	end
end

function gameState:mousepressed(x, y, b)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].mousepressed) == "function" then
				self.states[self.selectedState]:mousepressed(x, y, b)
			end
		end
	end
end

function gameState:mousereleased(x, y, b)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].mousereleased) == "function" then
				self.states[self.selectedState]:mousereleased(x, y, b)
			end
		end
	end
end

function gameState:resize(w, h)
	if self.selectedState then
		if self.states[self.selectedState] then
			if type(self.states[self.selectedState].resize) == "function" then
				self.states[self.selectedState]:resize(w, h)
			end
		end
	end
end

return gameState