--Creating window
love.filesystem.setIdentity("sokoban")
save = require "save"
configFile = save.newFile("config.txt")
res = love.window.getFullscreenModes()
local screenWidth, screenHeight, fullscreen, vsync = res[#res].width, res[#res].height, false, true
soundEnabled = true
showBackground = true
soundVolume = 0.8
musicVolume = 0.5
local exists = configFile:load()
if exists then
	--------------------------------------------------
	if configFile:exists("screenWidth") then
		local val = configFile:getValue("screenWidth")
			if type(val) == "number" then
				screenWidth = val
			end
	else
		configFile:addItem("screenWidth", screenWidth)
	end
	--------------------------------------------------
	if configFile:exists("screenHeight") then
		local val = configFile:getValue("screenHeight")
			if type(val) == "number" then
				screenHeight = val
			end
	else
		configFile:addItem("screenHeight", screenHeight)
	end
	--------------------------------------------------
	if configFile:exists("fullscreen") then
		local val = configFile:getValue("fullscreen")
			if type(val) == "boolean" then
				fullscreen = val
			end
	else
		configFile:addItem("fullscreen", fullscreen)
	end
	--------------------------------------------------
	if configFile:exists("vsync") then
		local val = configFile:getValue("vsync")
			if type(val) == "boolean" then
				vsync = val
			end
	else
		configFile:addItem("vsync", vsync)
	end
	--------------------------------------------------
	if configFile:exists("soundVolume") then
		local val = configFile:getValue("soundVolume")
			if type(val) == "number" then
				soundVolume = val
			end
	else
		configFile:addItem("soundVolume", soundVolume)
	end
	--------------------------------------------------
	if configFile:exists("musicVolume") then
		local val = configFile:getValue("musicVolume")
			if type(val) == "number" then
				musicVolume = val
			end
	else
		configFile:addItem("musicVolume", musicVolume)
	end
	--------------------------------------------------
	if configFile:exists("showBackground") then
		local val = configFile:getValue("showBackground")
			if type(val) == "boolean" then
				showBackground = val
			end
	else
		configFile:addItem("showBackground", showBackground)
	end
	--------------------------------------------------
else
	configFile:addItem("screenWidth", screenWidth)
	configFile:addItem("screenHeight", screenHeight)
	configFile:addItem("fullscreen", fullscreen)
	configFile:addItem("vsync", vsync)
	configFile:addItem("soundVolume", soundVolume)
	configFile:addItem("musicVolume", musicVolume)
	configFile:addItem("showBackground", showBackground)
	configFile:save()
end

--Screenshot folder
if not love.filesystem.exists("screenshots") then
	love.filesystem.createDirectory("screenshots")
end

love.window.setMode(screenWidth, screenHeight, {vsync = vsync, fullscreen = fullscreen})
love.window.setTitle("Sokoban")

screen = {
	width = love.window.getWidth(),
	height = love.window.getHeight(),
	fullscreen = fullscreen,
	vsync = vsync
}

function love.load()
	smooth_factor = 14
	--Loading classes
	gState = require "gameState"
	console = require "src/lib/console"
	require "src/lib/save"
	require "background"
	require "color"
	require "font"
	require "sound"
	require "tile"
	require "map"
	require "player"

	--READING MAPS
	maps = {}
	local files = love.filesystem.getDirectoryItems("src/map")
	for i,v in ipairs(files) do
		local s, e = string.find(v, ".", 1, true)
		if e then
			if v:sub(e) == ".txt" then
				maps[#maps + 1] = v:sub(1, s-1)
			end
		end
	end

	--GAMESTATES
	gState:newState("game")
	gState:newState("menu")

	--INITIALIZING
	console:init()
	console:setVisible(false)
	tile:load()
	player:load()
	sound:load()
	gState:load()
	background:load()
	if soundVolume > 0 then
		sound:play("song", musicVolume, false, 1, true)
	end

	gState:selectState("menu")
	
	map:load()

	love.graphics.setBackgroundColor(color[1])
	love.mouse.setVisible(false)
end

function love.update(dt)
	background:update(dt)
	gState:update(dt)
	console:update(dt)
end

function love.draw()
	background:draw()
	gState:draw()
	console:draw()

	if console:getVisible() then
		love.graphics.setColor(color[4])
		love.graphics.print(love.timer.getFPS(), 12, 12)
	end
end

function love.resize(w, h)
	screen.width = w
	screen.height = h
	game.tileSize = math.floor(math.min(w, h) * 0.08)
	player.stretchFactor = game.tileSize * 0.05
	gState:resize(w, h)
	font:resize(w, h)
	console:resize(w, h)
	background:resize(w, h)
end

function love.textinput(t)
	console:textinput(t)
end

function love.quit()
	if configFile then
		configFile:save()
	end
end

function love.keypressed(key)
	if key == "escape" then 
		if gState:getState() == "game" then
			gState:selectState("menu")
			sound:play("tick", soundVolume, false, 0.8)
		else
			--love.event.push("quit") 
		end
	end
	if key == "f1" then console:setVisible(not console:getVisible()) end
	if key == "f2" then
		local s = love.graphics.newScreenshot()
		local n = os.date("%x")
		s:encode("screenshots/"..os.time()..".png")
	end
	console:keypressed(key)

	gState:keypressed(key)
end

function love.mousepressed(x, y, key)
	gState:mousepressed(x, y, key)
end

function setAlpha(alpha)
	local r, g, b, a = love.graphics.getColor()
	love.graphics.setColor(r, g, b, a * alpha)
end

function copy(original)
    local t = {}
    for k, v in pairs(original) do
        if type(v) == 'table' then
            v = copy(v)
        end
        t[k] = v
    end
    return t
end

function verticalGradient(width, height, ...)
	local canvas = love.graphics.newCanvas(width, height)
	local vertices = {}
	for i,v in ipairs({...}) do
		local y = (height  / (#{...} - 1)) * (i - 1)
		vertices[#vertices + 1] = {
			0, y,
			0, 0,
			v[1], v[2], v[3]
		}
		vertices[#vertices + 1] = {
			width, y,
			0, 0,
			v[1], v[2], v[3]
		}
	end
	local mesh = love.graphics.newMesh(vertices, nil, "strip")

	love.graphics.setCanvas(canvas)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(mesh, 0, 0)
	love.graphics.setCanvas()

	return canvas
end

function horizontalGradient(width, height, ...)
	local canvas = love.graphics.newCanvas(width, height)
	local vertices = {}
	for i,v in ipairs({...}) do
		local x = (width  / (#{...} - 1)) * (i - 1)
		vertices[#vertices + 1] = {
			x, 0,
			0, 0,
			v[1], v[2], v[3]
		}
		vertices[#vertices + 1] = {
			x, height,
			0, 0,
			v[1], v[2], v[3]
		}
	end
	local mesh = love.graphics.newMesh(vertices, nil, "strip")

	love.graphics.setCanvas(canvas)
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.draw(mesh, 0, 0)
	love.graphics.setCanvas()

	return canvas
end