local save = {}
local sm = {__index = save}

local default = {
	file_name = "save.txt"
}
--local methods
local function stringToType(val)
	if tonumber(val) then --NUMBER
		return tonumber(val)
	else--BOOLEAN
		if val == "false" then
			return false
		elseif val == "true" then
			return true
		else--STRING
			return val
		end
	end
end

--Public methods
function save.newFile(name)
	name = name or default.file_name
	local f = {
		file_name = name,
		items = {}
	}
	return setmetatable(f, sm)
end

function save:load()
	local file, err = love.filesystem.newFile(self.file_name)
	if not love.filesystem.exists(self.file_name) then--File doesn't exist
		if err then print(err) end
		file:open("w")
		file:write("")
		file:close()
		return false
	else--File exists
		for line in love.filesystem.lines(self.file_name) do
			local s = line:find("=")
			local name = line:sub(0, s-1)
			local value = line:sub(s+1)
			self:addItem(name, value)
		end
		return true
	end
end	

function save:save()
	local data = ""
	for i,v in pairs(self.items) do
		data = data..v.name.."="..tostring(v.value).."\n"
	end
	print(data)
	local s = love.filesystem.write(self.file_name, data)
end

function save:addItem(name, value)
	name = tostring(name)
	if not self:exists(name) then
		local value_type = type(value)
		if value_type == "string" or value_type == "boolean" or value_type == "number" then
			self.items[name] = {}
			self.items[name].name = name
			local v = stringToType(value)
			if v == nil then
				print(("Item type '%s' not supported."):format(value_type))
			else
				self.items[name].value = v
			end
		else
			print(("Item type '%s' not supported."):format(value_type))
		end
	else
		print(("Item '%s' already exists"):format(name))
	end
end

function save:removeItem(name)
	name = tostring(name)
	if self:exists(name) then
		self.items[name] = nil
	else
		print(("Item '%s' not found"):format(name))
	end
end

function save:clearItems()
	self.items = {}
end

function save:getName(name)
	if self:exists(name) then
		return self.items[name].name
	else
		print(("Item '%s' not found"):format(name))
		return nil
	end
end

function save:getValue(name)
	name = tostring(name)
	if self:exists(name) then
		return self.items[name].value
	else
		print(("Item '%s' not found"):format(name))
		return nil
	end
end

function save:setValue(name, value)
	name = tostring(name)
	if self:exists(name) then
		self.items[name].value = value
	else
		print(("Item '%s' not found"):format(name))
	end
end

function save:countItems()
	local c = 0
	for i,v in pairs(self.items) do
		c = c + 1
	end
	return c
end

function save:exists(name) 
	name = tostring(name)
	if self.items[name] then
		return true
	else
		return false
	end
end

return save